#pragma once

#include "MemoryCell.h"

#include <iosfwd>
#include <stdexcept>
#include <vector>

/* A Heap is just a Vector of MemoryCells, but it has a maximum size (determined
  when it is initialized). Any attempt to grow beyond the maximum size of the
  Heap results in garbage collection--if there is still not enough memory,
  after the garbage collection, an OutOfMemoryException should be thrown. */

namespace gc {

class OutOfMemoryException : public std::exception {};

using Stack = std::vector<MemoryCell *>;

class Heap {
private:
  using SemiSpace = std::vector<MemoryCell>;
  SemiSpace memoryPool;

public:
  Heap(std::size_t);

  std::size_t capacity() const;
  std::size_t size() const;

  friend std::ostream &operator<<(std::ostream &, const Heap &);
  friend std::ostream &printGrading(std::ostream &, const Heap &);
  friend std::ostream &printGrading(std::ostream &, const Stack &,
                                    const Heap &);
};

std::ostream &operator<<(std::ostream &, const Heap &);
std::ostream &printGrading(std::ostream &, const Heap &);
} // namespace gc
